import pygame
from pygame.locals import *
from math import pi, sin, cos, sqrt
import random
import sys
from vector import Vector
from world import World
from enemies import Enemy1
from section import Section

class EditorSettings(object):
    def __init__(self, world):
        self.textures = False
        self.saveId = 0
        self.saveTimer = 0
        self.selectionMode = 0
        self.selectedIndex = 0
        self.selectedEnemy = 0
        self.selectedTexture = world.mapTextures.values()[0]
        self.warnLength = False
        self.warnAngles = False

try:
    from OpenGL.GL import *
    from OpenGL.GLU import *
except:
    print 'This game requires PyOpenGL'
    raise SystemExit

class Camera(object):
    def __init__(self, pos):
        self.pos = pos
        self.yrot = 0
        self.xrot = 0
    def forward(self, dist):
        facing = Vector(0,0,-1)
        facing = facing.rotate(-self.xrot,Vector(1,0,0))
        facing = facing.rotate(-self.yrot,Vector(0,1,0))
        self.pos += facing*dist
    def strafe(self, dist):
        right = Vector(1,0,0)
        right =right.rotate(-self.xrot,Vector(1,0,0))
        right = right.rotate(-self.yrot,Vector(0,1,0))
        self.pos += right*dist
    def rise(self, dist):
        up = Vector(0,1,0)
        up = up.rotate(-self.xrot,Vector(1,0,0))
        up = up.rotate(-self.yrot,Vector(0,1,0))
        self.pos += up*dist
    def rotateDown(self, angle):
        self.xrot += angle
        while self.xrot < 0: self.xrot += 2*pi
        while self.xrot >= 2*pi: self.xrot -= 2*pi
    def rotateRight(self, angle):
        self.yrot += angle
        while self.yrot < 0: self.yrot += 2*pi
        while self.yrot >= 2*pi: self.yrot -= 2*pi

def moveCamera(camera):
    if pygame.key.get_pressed()[K_a]: camera.strafe(-0.1)
    if pygame.key.get_pressed()[K_d]: camera.strafe(0.1)
    if pygame.key.get_pressed()[K_w]: camera.forward(0.1)
    if pygame.key.get_pressed()[K_s]: camera.forward(-0.1)
    if pygame.key.get_pressed()[K_r]: camera.rise(0.1)
    if pygame.key.get_pressed()[K_f]: camera.rise(-0.1)
    (rota,rotb) = pygame.mouse.get_rel()
    camera.rotateRight(rota/(2*pi*20))
    camera.rotateDown(rotb/(2*pi*20))

def recalcWorld(settings, world, idx):
    if len(world.map) > 1:
        first = max(idx-1, 0)
        last = min(idx+1, len(world.map))
        for s in world.map[first:last]: s.updateCoords()
        for s in world.map[first:last]: s.calculatePoints()
        settings.warnLength = False
        settings.warnAngles = False
        for s in world.map[1:-1]:
            a = s.prev.pos-s.pos
            b = s.next.pos-s.pos
            if a.norm() < 1 or b.norm() < 1:
                settings.warnLength = True
                continue
            if a.dot(b) / a.norm() / b.norm() > -0.707:
                settings.warnAngles = True

def modifyWorld(settings, camera, world):
    if settings.selectionMode == 2:
        s = world.map[settings.selectedIndex]
        if pygame.key.get_pressed()[K_UP]:
            facing = Vector(0,0,-1)
            facing = facing.rotate(-camera.xrot,Vector(1,0,0))
            facing = facing.rotate(-camera.yrot,Vector(0,1,0))
            s.pos += facing*0.1
            recalcWorld(settings, world, settings.selectedIndex)
        if pygame.key.get_pressed()[K_DOWN]:
            facing = Vector(0,0,-1)
            facing = facing.rotate(-camera.xrot,Vector(1,0,0))
            facing = facing.rotate(-camera.yrot,Vector(0,1,0))
            s.pos -= facing*0.1
            recalcWorld(settings, world, settings.selectedIndex)
        if pygame.key.get_pressed()[K_LEFT]:
            right = Vector(1,0,0)
            right = right.rotate(-camera.xrot,Vector(1,0,0))
            right = right.rotate(-camera.yrot,Vector(0,1,0))
            s.pos -= right*0.1
            recalcWorld(settings, world, settings.selectedIndex)
        if pygame.key.get_pressed()[K_RIGHT]:
            right = Vector(1,0,0)
            right = right.rotate(-camera.xrot,Vector(1,0,0))
            right = right.rotate(-camera.yrot,Vector(0,1,0))
            s.pos += right*0.1
            recalcWorld(settings, world, settings.selectedIndex)
        if pygame.key.get_pressed()[K_PAGEUP]:
            up = Vector(0,1,0)
            up = up.rotate(-camera.xrot,Vector(1,0,0))
            up = up.rotate(-camera.yrot,Vector(0,1,0))
            s.pos += up*0.1
            recalcWorld(settings, world, settings.selectedIndex)
        if pygame.key.get_pressed()[K_PAGEDOWN]:
            up = Vector(0,1,0)
            up = up.rotate(-camera.xrot,Vector(1,0,0))
            up = up.rotate(-camera.yrot,Vector(0,1,0))
            s.pos -= up*0.1
            recalcWorld(settings, world, settings.selectedIndex)

def writeText(posx, posy, text):
    glRasterPos2fv((posx, posy))
    #for c in text:
    #    glutBitmapCharacter(GLUT_BITMAP_8_BY_13, ord(c))


def drawScene(settings, camera, world):
    """draw the scene"""

    glLoadIdentity()
    glLineWidth(1.0)
    glColor3f(1,1,1)

    # Text
    glBindTexture(GL_TEXTURE_2D, 0)
    writeText(-1.6, 1.1, "Posicion: %.3f, %.3f, %.3f"%(camera.pos.coords()))
    writeText(0, 1., "%d:%d" % (world.duration() / 60, world.duration() % 60))
    writeText(-1.6, 1, "Rotacion Y: %.2f, Rotacion X: %.2f"%(camera.yrot*180/pi, camera.xrot*180/pi))
    if settings.saveTimer:
        glColor(0,1,0)
        writeText(-1.6, .9, settings.saveMessage)
        settings.saveTimer -= 1
    if settings.warnAngles:
        glColor3f(1,0,0)
        writeText(-1.6, -1.1, "Warning: There is one or more angles over 45 degress.")
    if settings.warnLength:
        glColor3f(1,0,0)
        writeText(-1.6, -1, "Warning: There is a section of length < 1.")

    # Draw currently selected texture
    glBindTexture(GL_TEXTURE_2D, settings.selectedTexture.texid)
    glColor(1,1,1)
    glBegin(GL_QUADS)
    glTexCoord2f(0,0)
    glVertex2f(1.4, 1)
    glTexCoord2f(0,1)
    glVertex2f(1.4, 1.2)
    glTexCoord2f(1,1)
    glVertex2f(1.6, 1.2)
    glTexCoord2f(1,0)
    glVertex2f(1.6, 1)
    glEnd()

    # Rotate according to camera
    glRotatef(camera.xrot*180/pi,1,0,0)
    glRotatef(camera.yrot*180/pi,0,1,0)

    # Draw targeting sphere
    #glColor3f(1,1,1)
    #glutWireSphere(.05, 8, 8);

    # Draw axis
    glBindTexture(GL_TEXTURE_2D, 0)
    glBegin(GL_LINES)
    glColor3f(1,0,0)
    glVertex3f(0,0,0)
    glVertex3f(.1,0,0)
    glColor3f(0,1,0)
    glVertex3f(0,0,0)
    glVertex3f(0,.1,0)
    glColor3f(0,0,1)
    glVertex3f(0,0,0)
    glVertex3f(0,0,-.1)
    glEnd()

    # Translate according to camera
    glTranslatef(-camera.pos.x, -camera.pos.y, -camera.pos.z)

    # Draw enemies
    glColor3f(0,0,1)
    for e in world.enemies:
        if settings.selectionMode == 4:
            if world.enemies.index(e) == settings.selectedEnemy:
                continue
        glTranslatef(e.pos.x, e.pos.y, e.pos.z)
        #glutWireSphere(.05, 8, 8);
        glTranslatef(-e.pos.x, -e.pos.y, -e.pos.z)
    if settings.selectionMode == 4 and settings.selectedEnemy < len(world.enemies):
        e = world.enemies[settings.selectedEnemy]
        glColor(1,0,0)
        glTranslatef(e.pos.x, e.pos.y, e.pos.z)
        #glutWireSphere(.05, 8, 8);
        glTranslatef(-e.pos.x, -e.pos.y, -e.pos.z)

    # Draw tunnel
    glColor3f(1,1,1)
    if len(world.map) > 1:
        if settings.selectionMode == 1:
            for s in world.map[:settings.selectedIndex]:
                s.draw()
            for s in world.map[settings.selectedIndex+1:-1]:
                s.draw()
            s = world.map[settings.selectedIndex]
            s.color = (1,0,0)
            s.next.color = (1,0,0)
            glColor3f(1,0,0)
            s.draw()
            s.color = (1,1,1)
            s.next.color = (1,1,1)
        elif settings.selectionMode == 2:
            for s in world.map[:-1]:
                s.draw()
            glColor3f(1,0,0)
            glLineWidth(5.0)
            glBegin(GL_LINE_STRIP)
            for p in world.map[settings.selectedIndex].points:
                glVertex3fv(p.coords())
            glVertex3fv(world.map[settings.selectedIndex].points[0].coords())
            glEnd()
        elif settings.selectionMode == 3:
            for s in world.map[:-1]:
                s.draw()
            glColor3f(1,0,0)
            glLineWidth(5.0)
            glBegin(GL_LINE_STRIP)
            for p in world.map[settings.selectedIndex].points:
                glVertex3fv(p.coords())
            glVertex3fv(world.map[settings.selectedIndex].points[0].coords())
            glEnd()
            glBegin(GL_LINE_STRIP)
            for p in world.map[settings.selectedIndex+1].points:
                glVertex3fv(p.coords())
            glVertex3fv(world.map[settings.selectedIndex+1].points[0].coords())
            glEnd()
            glBegin(GL_LINES)
            for p, q in zip(world.map[settings.selectedIndex].points, world.map[settings.selectedIndex+1].points):
                glVertex3fv(p.coords())
                glVertex3fv(q.coords())
            glEnd()
        else:
            for s in world.map[:-1]:
                s.draw()

def mouseClicked(event, settings, camera, world):
    if len(world.map) > 0:
        s = Section (camera.pos, world.map[-1])
    else:
        s = Section (camera.pos, None)
    s.color = (1,1,1)
    s.texture = settings.selectedTexture
    world.map.append(s)
    recalcWorld(settings, world, len(world.map)-1)

def keyPressed(event, settings, world):
    if event.key == K_t:
        settings.textures = not settings.textures
        if settings.textures:
            glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
            glDisable(GL_TEXTURE_2D)
        else:
            glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
            glEnable(GL_TEXTURE_2D)
    elif event.key == K_F5:
        settings.saveId += 1
        while settings.saveId < 1000:
            saveFile = "levels/level" + str(settings.saveId) + ".txt"
            if world.save(saveFile):
                break
            settings.saveId += 1
        if settings.saveId < 1000:
            settings.saveMessage = "Level saved to file " + saveFile
            settings.saveTimer = 120
    elif event.key >= K_0 and event.key <= K_4:
        settings.selectionMode = event.key - K_0
        if settings.selectionMode == 1 or settings.selectionMode == 3:
            if settings.selectedIndex == len(world.map)-1:
                settings.selectedIndex -= 1
    elif event.key == K_COMMA:
        if settings.selectionMode >= 1 and settings.selectionMode <= 3:
            if settings.selectedIndex > 0:
                settings.selectedIndex -= 1
        elif settings.selectionMode == 4:
            if settings.selectedEnemy > 0:
                settings.selectedEnemy -= 1
    elif event.key == K_PERIOD:
        if settings.selectionMode == 1 or settings.selectionMode == 3:
            if settings.selectedIndex < len(world.map)-2:
                settings.selectedIndex += 1
        elif settings.selectionMode == 2:
            if settings.selectedIndex < len(world.map)-1:
                settings.selectedIndex += 1
        elif settings.selectionMode == 4:
            if settings.selectedEnemy < len(world.enemies)-1:
                settings.selectedEnemy += 1
    elif pygame.key.get_pressed()[K_DELETE]:
        if settings.selectionMode == 4:
            if settings.selectedEnemy < len(world.enemies):
                del world.enemies[settings.selectedEnemy]
    elif pygame.key.get_pressed()[K_INSERT]:
        if settings.selectionMode == 2:
            i = 0
            section = settings.selectedIndex
            while i < len(world.enemies):
                if world.map.index(world.enemies[i].section) >= section:
                    break
                i += 1
            if i == len(world.enemies) or world.map.index(world.enemies[i].section) > section:
                e = Enemy1(world.map[section], 0, 0, None, world)
                world.enemies.insert(i, e)
                if settings.selectedEnemy >= i:
                    settings.selectedEnemy += 1
    elif pygame.key.get_pressed()[K_SPACE]:
        if settings.selectionMode == 2:
            s = world.map[settings.selectedIndex]
            s.texture = settings.selectedTexture
    elif pygame.key.get_pressed()[K_LEFTBRACKET]:
        if settings.selectedTexture:
            i = world.mapTextures.values().index(settings.selectedTexture)
            settings.selectedTexture = world.mapTextures.values()[(i-1) % len(world.mapTextures)]
        else:
            settings.selectedTexture = world.mapTextures.values()[0]
    elif pygame.key.get_pressed()[K_RIGHTBRACKET]:
        if settings.selectedTexture:
            i = world.mapTextures.values().index(settings.selectedTexture)
            settings.selectedTexture = world.mapTextures.values()[(i+1) % len(world.mapTextures)]
        else:
            settings.selectedTexture = world.mapTextures.values()[0]
    elif pygame.key.get_pressed()[K_l]:
        if settings.selectionMode == 1 or settings.selectionMode == 3:
            before = world.map[settings.selectedIndex]
            after = before.next
            s = Section((before.pos + after.pos)/2, before)
            s.color = (1,1,1)
            after.setPrev(s)
            world.map.insert(settings.selectedIndex+1, s)
            recalcWorld(settings, world, settings.selectedIndex+1)
    elif pygame.key.get_pressed()[K_g]:
        pygame.event.set_grab(not pygame.event.get_grab())

def main():
    """run the game"""

    #initialize pygame and setup an opengl display
    pygame.init()
    pygame.display.set_mode((640,480), OPENGL|DOUBLEBUF)
    pygame.mouse.set_visible(False)
    glEnable(GL_DEPTH_TEST)        #use our zbuffer
    glDepthFunc(GL_LEQUAL)
    textures = True
    glEnable(GL_TEXTURE_2D)        # Enable texture mapping.

    print glGetString(GL_VENDOR)
    print glGetString(GL_RENDERER)
    print glGetString(GL_VERSION)
    print glGetString(GL_EXTENSIONS)

    #setup the camera
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45.0,640/480.0,0.1,100.0)    #setup lens
    glTranslatef(0.0, 0, -3.0)
    glMatrixMode(GL_MODELVIEW)

    clock = pygame.time.Clock()
    camera = Camera(Vector(0, 0, 0))
    world = World(640,480,45.0)
    if len(sys.argv) > 1:
        try:
            world.load('levels/'+sys.argv[1])
            del world.map[-1]
        except:
            print
            print "Level could not be loaded (file doesn't exist)"
            sys.exit(1)
    pygame.event.set_grab(1)
    settings = EditorSettings(world)

    #glutInit([])
    while 1:
        #check for events
        event = pygame.event.poll()
        while event:
            if event.type == QUIT or (event.type == KEYDOWN and event.key == K_ESCAPE):
                sys.exit(0)
            elif event.type == MOUSEBUTTONDOWN:
                mouseClicked(event, settings, camera, world)
            elif event.type == KEYDOWN:
                keyPressed(event, settings, world)
            event = pygame.event.poll()

        #clear screen
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
        #move camera
        moveCamera(camera)
        #modify world
        modifyWorld(settings, camera, world)
        #draw scene
        drawScene(settings, camera, world)
        pygame.display.flip()
        clock.tick(60)


if __name__ == '__main__': main()
