DAMAGE_FLASH_TIME = 35 # Time (in ticks) for the red flash when player is hit
DAMAGE_FLASH_INTENSITY = 0.8
MAX_HEAR_DIST = 10.0 # Maximum distance of heard plasma bolts

from OpenGL.GL import *
from OpenGL.GLU import *
from particles import ParticleEngine
from data import filepath
import pygame.mixer

fx = dict(damageTimer = 0, music=True, sound=True, fx=True, damageColor =(0,0,0))
partEngine = ParticleEngine()


sound_available = True
try:
    # Sound FX
    pygame.mixer.init()
    laser_sound = pygame.mixer.Sound(filepath ("laser.wav"))
    plasma_sound = pygame.mixer.Sound(filepath ("plasmablast.wav"))
    scratch_sound = pygame.mixer.Sound(filepath ("scratch.wav"))
    collision_sound = pygame.mixer.Sound(filepath ("collision.wav"))
    puc_sound =  pygame.mixer.Sound(filepath("puc.wav"))
    poc_sound =  pygame.mixer.Sound(filepath("poc.wav"))
    explosion_sound =  pygame.mixer.Sound(filepath("explosion.wav"))

except pygame.error:
    sound_available = False

def puc():
    if sound_available and fx['sound']:
        puc_sound.play()

def poc():
    if sound_available and fx['sound']:
        poc_sound.play()

def fire():
    if sound_available and fx['sound']:
        laser_sound.play()

def plasma(dist):
    if sound_available and fx['sound']:
        vol = (MAX_HEAR_DIST-dist)/MAX_HEAR_DIST
        if vol > 0:
            ch = pygame.mixer.find_channel()
            if not ch: return # No channels available, don't care
            ch.set_volume (vol)
            ch.play (plasma_sound)

def scratch():
    if sound_available and fx['sound']:
        scratch_sound.play()

def damage():
    """Initialize red flash effect, used when player is hit"""
    if fx['fx']:
        fx['damageTimer'] = DAMAGE_FLASH_TIME
        fx['damageColor'] = (1,0,0)

def shieldDamage():
    """Initialize red flash effect, used when player is hit"""
    if fx['fx']:
        fx['damageTimer'] = DAMAGE_FLASH_TIME
        fx['damageColor'] = (0.5,0.5,1)

def crash():
    if sound_available and fx['sound']:
        collision_sound.play()

def laserBounce(pos):
    if fx['fx']:
        partEngine.addCloud (pos, 10, 0.03)

def enemyDamageSparks(pos):
    if fx['fx']:
        partEngine.addCloud (pos, 10, 0.03, colors=((0.5,0.5,0.5),(0.5,0.5,0.75),(0.75,0.75,1)))

def plasmaExplosion(pos):
    if fx['fx']:
        partEngine.addCloud (pos, 10, 0.13, speed=0.05, life=1.5)

def burn(pos):
    if fx['fx']:
        partEngine.addCloud (pos, 1, 0.04, speed=0.003, life=4)

def smallExplosion(pos, dist):
    partEngine.addCloud (pos, 10, 0.13, life=1.5)
    if fx['fx']:
        partEngine.addCloud (pos, 10, 0.05, speed=0.03, life=2)
    if sound_available and fx['sound']:
        vol = (MAX_HEAR_DIST-dist)/MAX_HEAR_DIST
        if vol > 0:
            ch = pygame.mixer.find_channel()
            if not ch: return # No channels available, don't care
            ch.set_volume (vol)
            ch.play (explosion_sound)

def bigExplosion(pos, dist):
    partEngine.addCloud (pos, 30, 0.13, life=1.5)
    if fx['fx']:
        partEngine.addCloud (pos, 50, 0.05, speed=0.03, life=2)
    if sound_available and fx['sound']:
        vol = (MAX_HEAR_DIST-dist)/MAX_HEAR_DIST
        if vol > 0:
            ch = pygame.mixer.find_channel()
            if not ch: return # No channels available, don't care
            ch.set_volume (vol)
            ch.play (explosion_sound)

def tick(world):
    """Update timers related to effects"""
    if fx['fx']:
        if fx.get('damageTimer', 0):
            fx['damageTimer'] -= 1
        partEngine.pump(world)

def drawHud():
    """draw HUD effects. Might modify bound texture, blend function"""
    if fx['fx']:
        if fx.get ('damageTimer', 0):
            glBindTexture(GL_TEXTURE_2D, 0)
            glBlendFunc (GL_SRC_ALPHA, GL_ONE)
            glBegin(GL_QUADS)
            color = fx['damageColor']
            glColor4f(color[0],color[1],color[2],DAMAGE_FLASH_INTENSITY*fx['damageTimer']/DAMAGE_FLASH_TIME)
            glVertex2f(0, 0)
            glVertex2f(1, 0)
            glVertex2f(1, 1)
            glVertex2f(0, 1)
            glEnd()
            fx['damageTimer'] -= 1

def stopMusic():
    if sound_available and fx['music']:
        pygame.mixer.music.stop()

def playMusic(song):
    if sound_available and fx['music']:

        songs = {'menu': "mindprobe_-_puerto_23.ogg",
                 'level0': "mindprobe_-_fluorobox.ogg",
                 'level1': "mindprobe_-_realms_edge.ogg",
                 'level2': "mindprobe_-_push_edi.ogg",
                 'level3': "mindprobe_-_distopia.ogg",
                 'aurora': "Aurora.ogg"}
        if pygame.mixer.music.get_busy():
            pygame.mixer.music.fadeout(200)
        try:
            pygame.mixer.music.load(filepath('music/' + songs[song]))
            pygame.mixer.music.play(-1)
        except pygame.error:
            print "You seem to be missing the music files.\n"

def fadeOut():
    if sound_available and fx['music']:
        if pygame.mixer.music.get_busy():
            pygame.mixer.music.fadeout(200)

def loadConfig():
    try:
        f = open('sarasa.config')
        config = eval(f.readline())
        fx.update(config)
    except:
        print "No config file found, using defaults."

def saveConfig():
    keystosave = ['music', 'sound', 'fx']
    config = {}
    for k in keystosave:
        config[k] = fx[k]
    try:
        f = open('sarasa.config', 'w')
        f.write(str(config) + '\n')
        f.close()
    except:
        print "Could not write config file\n"
