from OpenGL.GL import *
from OpenGL.GLU import *
from ship import Ship
from vector import Vector
from shot import Shot, PlasmaBlast
import effects
import pygame

import math

# Should be imported from world, but there is a circular dependency and
# There is no time :) Hardcode!
FPS = 60.0

class Hero (Ship):
    fullHealth = 6.0
    fullPower = 1.0
    powerRecharge = fullPower / 10 / FPS # 10 seconds
    fullShield = 1.0
    shieldTicks = 25
    shieldSteps = 15
    shieldStep = fullShield / shieldSteps # Shield update is discrete. intentionally
    shieldPower = shieldTicks * powerRecharge * 1.1 # Recharge drains power slowly
    baseSpeed = 0.04
    burnerSpeed = baseSpeed*1.6
    burnerPower = fullPower / 4 / FPS # 4 seconds (if not recharging)
    # Position of the laser cannons, relative to the ship center
    gun_lx = -0.05
    gun_rx = 0.05
    gun_y = -0.02
    gun_reloadtime = 15
    gun1Power = gun_reloadtime * powerRecharge # unlimited (borderline) iff recharging 100%
    gun2_reloadtime = 120
    gun2Power = gun2_reloadtime * powerRecharge * 2
    friction = 0.5

    collisionDamage = 0.3 # with walls

    def __init__ (self, section, world):
        Ship.__init__ (self, section)
        self.lightPosition = [0, 0, 0, 1]
        self.camera = Vector (0, 0, -1) # Camera direction
        self.camerasec = self.section
        self.cameradist = 0
        #self.t = 0 # For interpolation
        #self.omega = 0 # For interpolation
        self.world = world
        self.weapontimer = 0
        self.weapon2timer = 0
        self.health = self.fullHealth
        self.power = self.fullPower
        self.speed = self.baseSpeed
        self.shield = self.fullShield
        self.burnerEnabled = False
        self.shieldTimer = 0
        self.pushed = Vector (0,0,0) # Speed at which we are being pushed
        self.pause = False
        self.velx = 0
        self.vely = 0
        self.accelx = 0
        self.accely = 0
        self.posx = 0
        self.posy = 0

    def fire(self, mousePos):
        """ Where!? Where!? """
        if self.weapontimer > 0: return
        if self.power < self.gun1Power: return

        h = (self.world.width / self.world.height)/2
        rdir = Vector(-(mousePos[0] / self.world.height - h),
                      -(mousePos[1] / self.world.height - 0.5), 0.5)
        rdir.normalize()
        
        # Ideally, (camera; right; up) would be an orthogonal base. But it is not
        # Adapt, adopt, and project onto plane.
        u = self.section.up # Wrong!
        u -= self.camera * (self.camera.dot (u)) # Right (almost)!
        u = u.normalize ()
        # OK, u is now right, r is easy
        r = u.cross(self.camera)
        
        realdir = self.camera*rdir.z + u*rdir.y + r*rdir.x
        b = Shot(self.pos + r * self.gun_lx + u * self.gun_y, realdir, self.section, True)
        self.world.shots.append (b)
        b = Shot(self.pos + r * self.gun_rx + u * self.gun_y, realdir, self.section, True)
        self.world.shots.append (b)
        
        effects.fire()
        
        self.power -= self.gun1Power

        # Fire rate. goes down when the timer goes up
        self.weapontimer = self.gun_reloadtime

    def firePowerShot(self, mousePos):
        if self.weapon2timer > 0: return
        if self.power < self.gun2Power: return
        
        h = (self.world.width / self.world.height)/2
        rdir = Vector(-(mousePos[0] / self.world.height - h),
                      -(mousePos[1] / self.world.height - 0.5), 0.5)
        rdir.normalize()
        
        # Ideally, (camera; right; up) would be an orthogonal base. But it is not
        # Adapt, adopt, and project onto plane.
        u = self.section.up # Wrong!
        u -= self.camera * (self.camera.dot (u)) # Right (almost)!
        u = u.normalize ()
        # OK, u is now right, r is easy
        r = u.cross(self.camera)
        
        realdir = self.camera*rdir.z + u*rdir.y + r*rdir.x
        b = PlasmaBlast(self.pos, realdir, self.section, self.world, self.world.textures['powerblast'], True)
        # Must be faster than the player with afterburner on
        b.speed = 0.07
        self.world.shots.append (b)

        effects.plasma(0.1)

        self.power -= self.gun2Power

        # Fire rate. goes down when the timer goes up
        self.weapon2timer = self.gun2_reloadtime

    def enableBurner(self):
        self.burnerEnabled = True

    def disableBurner(self):
        self.burnerEnabled = False

    def rechargeShield(self):
        """should be called every tick, check if shiled needs to recharge and do it"""
        if self.shieldTimer > 0:
            self.shieldTimer -= 1
            return
        if self.shield==self.fullShield: return
        if self.power < self.shieldPower: return # No power to recharge
        # Really recharge
        self.shield = min (self.shield + self.shieldStep, self.fullShield)
        self.shieldTimer = self.shieldTicks
        self.power -= self.shieldPower
        
    
    def tick(self):
        #self.pos += self.pushed
        if not self.pause:
            self.forward (self.speed)
            self.speed *= 0.95
            if self.speed < self.baseSpeed:
                self.speed = self.baseSpeed
        self.velx = self.velx * self.friction + self.accelx
        self.vely = self.vely * self.friction + self.accely
        self.accelx = 0
        self.accely = 0
        self.strafe(self.velx, self.vely)
        self.posx += self.velx
        self.posy += self.vely
        #self.pushed *= 0.95 # Slowly stabilize
        if self.weapontimer > 0:
            self.weapontimer -= 1
        if self.weapon2timer > 0:
            self.weapon2timer -= 1
        # Shield
        self.rechargeShield()
        # Afterburner
        if self.burnerEnabled:
            if self.power >= self.burnerPower:
                self.increaseSpeed()
                self.power -= self.burnerPower
            else:
                self.disableBurner()
        # Power
        self.power = min (self.power + self.powerRecharge, self.fullPower)

    def positionCamera (self):
        """ Set the camera and lights at our hero's point of view """

        # Catmull
        MAXDRIFT = self.speed * 8
        sec = self.camerasec
        d = self.cameradist
        cpos = sec.id+d
        ppos = self.section.id+self.dist
        
        if cpos > ppos:
            pass
        elif ppos - cpos < MAXDRIFT:
            self.cameradist = self.dist
            self.camerasec = self.section
        else:
            self.cameradist += MAXDRIFT # Limit camera displacement
        while self.cameradist > 1:
            self.camerasec = self.camerasec.next
            self.cameradist -= 1
        sec = self.camerasec
        d = self.cameradist

        secs = [sec.prev, sec, sec.next, sec.next.next]
        if secs[0] is None:
            secs[0] = sec
        if secs[3] is None:
            secs[3] = secs[2]
        x = catmull(d, secs[0].pos.x, secs[1].pos.x, secs[2].pos.x, secs[3].pos.x)
        y = catmull(d, secs[0].pos.y, secs[1].pos.y, secs[2].pos.y, secs[3].pos.y)
        z = catmull(d, secs[0].pos.z, secs[1].pos.z, secs[2].pos.z, secs[3].pos.z)
        xp = catmull(d+0.001, secs[0].pos.x, secs[1].pos.x, secs[2].pos.x, secs[3].pos.x)-x
        yp = catmull(d+0.001, secs[0].pos.y, secs[1].pos.y, secs[2].pos.y, secs[3].pos.y)-y
        zp = catmull(d+0.001, secs[0].pos.z, secs[1].pos.z, secs[2].pos.z, secs[3].pos.z)-z
        self.camera = Vector (xp, yp, zp).normalize()

        self.camera = self.camera.rotate(self.velx, self.section.up)
        self.camera = self.camera.rotate(-self.vely, self.section.right)
        # Approximation without rotations. Use if last part is too slow.
        #self.camera += self.section.up * self.vely
        #self.camera += self.section.right * self.velx
        #self.camera = self.camera.normalize()

        # Jumpy (non-interpolated):
        #xp = self.section.path_vector.x
        #yp = self.section.path_vector.y
        #zp = self.section.path_vector.z
        
        # Linear
        #if self.t > 0:
        #    self.camera += (self.section.path_vector.normalize() - self.camera) / self.t
        #    self.t -= 1
        #elif (self.section.path_vector.normalize() - self.camera).norm() > 0.0000001:
        #    self.t = 64
        #self.camera = self.camera.normalize()
        #xp, yp, zp = self.camera.x, self.camera.y, self.camera.z, 

        # Spherical (looks the same as linear :( )
        #sin = math.sin
        #if self.t > 0:
        #    self.camera = (self.oldcam*sin(self.omega*self.t) + 
        #        self.section.path_vector.normalize()*sin(self.omega*(1-self.t))) / sin(self.omega)
        #    self.t -= (1./64)
        #elif (self.section.path_vector.normalize() - self.camera).norm() > 0.0000001:
        #    self.t = 1
        #    costheta = min (self.section.path_vector.normalize().dot (self.camera), 1.0)
        #    self.omega = math.acos (costheta)
        #    self.oldcam = Vector (self.camera.x, self.camera.y, self.camera.z)
        #
        #self.camera = self.camera.normalize()
        #xp, yp, zp = self.camera.x, self.camera.y, self.camera.z, 

        
        glLoadIdentity()
        #glTranslatef(0, 0, 0.018)
        #glTranslatef(-x, -y, -z)
        # Why were we using xp instead of self.camera.x?
        #gluLookAt(self.pos.x, self.pos.y, self.pos.z,
                 #self.pos.x + xp, self.pos.y + yp, self.pos.z + zp,
                 #self.section.up.x, self.section.up.y, self.section.up.z)
        gluLookAt(self.pos.x, self.pos.y, self.pos.z,
                  self.pos.x + self.camera.x, self.pos.y + self.camera.y, self.pos.z + self.camera.z,
                  self.section.up.x, self.section.up.y, self.section.up.z)

        self.lightPosition[0] = self.pos.x
        self.lightPosition[1] = self.pos.y
        self.lightPosition[2] = self.pos.z
        glLightfv(GL_LIGHT0, GL_POSITION, self.lightPosition)

    def takeDamage(self, damage, skipShield=False):
        if self.shield < self.shieldStep or skipShield:
            self.health -= damage
            effects.damage() # red flash when hit
        else:
            self.shield -= damage
            if self.shield < 0:
                self.health += self.shield #Damage not absorbed by shields
                self.shield = 0 
                effects.damage() # red flash when hit
            else:
                effects.shieldDamage() # blue flash when hit

    def collidedWall (self):
        self.takeDamage(self.collisionDamage, skipShield=True)
        effects.scratch()
        #vec = self.section.path_vector
        #center = self.section.pos - self.pos
        # center = self.section.pos + vec * self.dist - self.pos
        # But due to artifacts in calculations of dist, center may not be
        # fully perpendicular to path_vector. Let's correct:
        #center -= vec * center.dot(vec)
        #self.pushed += center / 80
        #self.pos += self.pushed * 5
        self.accelx -= .4*self.posx/self.section.rad
        self.accely -= .4*self.posy/self.section.rad

    def crashWith (self, enemy):
       self.takeDamage (self.crashDamage(enemy), skipShield=True)
       dist = self.pos - enemy.pos # Away vector
       #self.pushed += dist / 12
       self.accely += dist.dot(self.section.up)
       self.accelx += dist.dot(self.section.right)
       effects.crash()

    def crashDamage (self, enemy):
        """Amount of damage taken when colliding with enemy"""
        return 1.2

    def accel(self, dx, dy):
        self.accelx += dx
        self.accely += dy
        
    def increaseSpeed(self):
        self.speed += 0.05*self.burnerSpeed

#Auxiliar, for path interpolation

#M11 = 0
M12 = 1
#M13 = 0
#M14 = 0
M21 = -.5
#M22 = 0
M23 = 0.5
#M24 = 0
M31 = 1
M32 = -2.5
M33 = 2
M34 = -.5
M41 = -.5
M42 = 1.5
M43 = -1.5
M44 = .5
def catmull(x, v0, v1, v2, v3):
    c1 =          M12*v1
    c2 = M21*v0          + M23*v2
    c3 = M31*v0 + M32*v1 + M33*v2 + M34*v3
    c4 = M41*v0 + M42*v1 + M43*v2 + M44*v3
    return (((c4*x + c3)*x +c2)*x + c1)

