from OpenGL.GL import *
from OpenGL.GLU import *
from pygame.locals import *
import pygame
from vector import Vector
from math import pi
import effects

increments = [(0, 0, 0), (0, 0, 0), (0, 0, 0),
              (-15, 0, -5), (0, 0, -5), (10, 0, -5), (10, 0, -5),
              (20, 3, -5), (20, 3, -5), (20, 3, 25), (20, 3, 25),
              (0, 10, 25), (0, 10, 0), (0, 10, 0), (0, 10, 0),
              (0, 20, 0), (0, 20, 20), (0, 0, 0), (0, 0, 0), (0, 0, 0)]

def points():
    p = Vector(0, -0.7, 0)
    forward = Vector(0, 0, 1)
    right = Vector(1, 0, 0)
    up = Vector (0, 1, 0)
    res = []
    for i in increments:
        res.append(((p-right).coords(), (p+right).coords()))
        p = p + forward
        forward, up, right = (forward.rotate (pi*i[0] / 180.0, right).rotate (pi*i[2] / 180.0, up),
                              up.rotate (pi*i[0] / 180.0, right).rotate (pi*i[1] / 180.0, forward),
                              right.rotate (pi*i[1] / 180.0, forward).rotate (pi*i[2] / 180.0, up))
    return res

def main(width, height, tex):
    glDisable(GL_DEPTH_TEST)
    glDisable(GL_LIGHTING)
    glColor4f(1, 1, 1, 0.5)
    glEnable(GL_BLEND)
    glEnable(GL_TEXTURE_2D)
    position = 0.9
    clock = pygame.time.Clock()
    skipped = False
    delay = 0
    effects.playMusic('aurora')
    while delay < 240 and not skipped:
        # Blank screen
        glClear(GL_COLOR_BUFFER_BIT)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        # Starry background
        glBlendFunc (GL_SRC_ALPHA, GL_ONE)
        glColor4f(1, 1, 1, 1)
        glBindTexture (GL_TEXTURE_2D, tex['menubg'].texid)
        glBegin(GL_QUADS)
        glTexCoord2f(0, 0)
        glVertex3f(-1.2, 1, 0)
        glTexCoord2f(0, 1)
        glVertex3f(1.2, 1, 0)
        glTexCoord2f(1, 1)
        glVertex3f(1.2, -2.2, 0)
        glTexCoord2f(1, 0)
        glVertex3f(-1.2, -2.2, 0)
        glEnd()
        # 'Press escape to skip intro' sign
        glColor4f(1, 1, 1, 1)
        glBindTexture (GL_TEXTURE_2D, tex['skip'].texid)
        glBegin(GL_QUADS)
        glTexCoord2f(1, 0)
        glVertex3f(-0.5, 0.93, 0)
        glTexCoord2f(0, 0)
        glVertex3f(-1, 0.93, 0)
        glTexCoord2f(0, 1)
        glVertex3f(-1, 1, 0)
        glTexCoord2f(1, 1)
        glVertex3f(-0.5, 1, 0)
        glEnd()
        if delay < 120:
            z = 0
            alpha = max(min(1-(delay-60)/50.0, 1), 0)
            glBindTexture (GL_TEXTURE_2D, tex['logo'].texid)
            glColor4f(1, 1, 1, alpha)
            glBegin(GL_QUADS)
            glTexCoord2f(1, 0.5)
            glVertex3f(1.2, -0.7, z)
            glTexCoord2f(0, 0.5)
            glVertex3f(-1.2, -0.7, z)
            glTexCoord2f(0, 1)
            glVertex3f(-1.2, 0.7, z)
            glTexCoord2f(1, 1)
            glVertex3f(1.2, 0.7, z)
            glEnd()
        else:
            z = 0
            alpha = min(1-(delay-60-120)/60.0, 1)
            glBindTexture (GL_TEXTURE_2D, tex['logo'].texid)
            glColor4f(1, 1, 1, alpha)
            glBegin(GL_QUADS)
            glTexCoord2f(1, 0)
            glVertex3f(1.5, -0.5, z)
            glTexCoord2f(0, 0)
            glVertex3f(-1.5, -0.5, z)
            glTexCoord2f(0, 0.5)
            glVertex3f(-1.5, 0.5, z)
            glTexCoord2f(1, 0.5)
            glVertex3f(1.5, 0.5, z)
            glEnd()
        delay += 1
        event = pygame.event.poll()
        while event.type != NOEVENT:
            if event.type == KEYDOWN and event.key == K_ESCAPE:
                skipped = True
            event = pygame.event.poll()
        pygame.display.flip()
        clock.tick(20)
    #
    # Show twisted marquee
    #
    while tex['marquee'].texid != 0 and position > -0.2 and not skipped:
        # Blank screen
        glClear(GL_COLOR_BUFFER_BIT)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        # Starry background
        glBlendFunc (GL_SRC_ALPHA, GL_ONE)
        glColor4f(1, 1, 1, 1)
        glBindTexture (GL_TEXTURE_2D, tex['menubg'].texid)
        glBegin(GL_QUADS)
        glTexCoord2f(0, 0)
        glVertex3f(-1.2, 1, 0)
        glTexCoord2f(0, 1)
        glVertex3f(1.2, 1, 0)
        glTexCoord2f(1, 1)
        glVertex3f(1.2, -2.2, 0)
        glTexCoord2f(1, 0)
        glVertex3f(-1.2, -2.2, 0)
        glEnd()
        # 'Press escape to skip intro' sign
        glColor4f(1, 1, 1, 1)
        glBindTexture (GL_TEXTURE_2D, tex['skip'].texid)
        glBegin(GL_QUADS)
        glTexCoord2f(1, 0)
        glVertex3f(-0.5, 0.93, 0)
        glTexCoord2f(0, 0)
        glVertex3f(-1, 0.93, 0)
        glTexCoord2f(0, 1)
        glVertex3f(-1, 1, 0)
        glTexCoord2f(1, 1)
        glVertex3f(-0.5, 1, 0)
        glEnd()
        # Twisted marquee
        glBlendFunc (GL_SRC_ALPHA, GL_ONE)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(45, width/height, 0.1, 40.0)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        gluLookAt (0, 0, 0, 0, -.3, 1, 0, 1, 0)
        alpha = 3.2 + 16 * (min(position, -0.15))
        glColor4f(1, 1, 1, alpha)
        glBindTexture (GL_TEXTURE_2D, tex['marquee'].texid)
        glBegin(GL_QUAD_STRIP)
        pos = position
        for p in points():
            glTexCoord2f(1, max(pos,0))
            glVertex3fv(p[0])
            glTexCoord2f(0, max(pos,0))
            glVertex3fv(p[1])
            pos = min (pos + 0.07, 1)
        glEnd()
        pygame.display.flip()
        position -= 0.00025
        clock.tick(50)
        event = pygame.event.poll()
        while event.type != NOEVENT:
            if event.type == KEYDOWN and event.key == K_ESCAPE:
                skipped = True
            event = pygame.event.poll()
    #
    # Pan down, enter planet.
    #
    position = -2.2
    while position <= -1 and not skipped:
        # Blank screen
        glClear(GL_COLOR_BUFFER_BIT)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        # Starry background
        glBlendFunc (GL_SRC_ALPHA, GL_ONE)
        glColor4f(1, 1, 1, 1)
        glBindTexture (GL_TEXTURE_2D, tex['menubg'].texid)
        glBegin(GL_QUADS)
        glTexCoord2f(0, 0)
        glVertex3f(-1.2, position + 3.2, 0)
        glTexCoord2f(0, 1)
        glVertex3f(1.2, position + 3.2, 0)
        glTexCoord2f(1, 1)
        glVertex3f(1.2, position, 0)
        glTexCoord2f(1, 0)
        glVertex3f(-1.2, position, 0)
        glEnd()
        # 'Press escape to skip intro' sign
        glColor4f(1, 1, 1, 1)
        glBindTexture (GL_TEXTURE_2D, tex['skip'].texid)
        glBegin(GL_QUADS)
        glTexCoord2f(1, 0)
        glVertex3f(-0.5, 0.93, 0)
        glTexCoord2f(0, 0)
        glVertex3f(-1, 0.93, 0)
        glTexCoord2f(0, 1)
        glVertex3f(-1, 1, 0)
        glTexCoord2f(1, 1)
        glVertex3f(-0.5, 1, 0)
        glEnd()
        pygame.display.flip()
        position += 0.0025
        clock.tick(60)
        event = pygame.event.poll()
        while event.type != NOEVENT:
            if event.type == KEYDOWN and event.key == K_ESCAPE:
                skipped = True
            event = pygame.event.poll()
    effects.fadeOut()
