from OpenGL.GL import *
from OpenGL.GLU import *
from pygame.locals import *
import pygame
import pygame.mixer
from data import filepath
from texloader import LoadGLTextures
import tunnel
import effects
import about

tex = {}

class Item(object):
    spacing = 1.08
    def __init__(self, x, y, w, h, idx):
        self.x = x
        self.y = y
        self.w = w
        self.h = h
        self.idx = idx
        self.selected = False
    def redraw(self):
        dx, dy = self.selected and (0.2, 0.04) or (0, 0)
        if self.selected:
            glColor4d(0, 213, 255, 255)
        else:
            glColor4f(0, 0.54, 0.74, 1)
        glBegin(GL_QUADS)
        glTexCoord2f(0, 1 - self.idx / 16.0)
        glVertex2f(self.x - dx, self.y + 1 - self.h * self.idx * self.spacing + dy)
        glTexCoord2f(0.3125, 1 - self.idx / 16.0)
        glVertex2f(self.x + self.w + dx, self.y + 1 - self.h * self.idx * self.spacing + dy)
        glTexCoord2f(0.3125, 1 - (self.idx + 1) / 16.0)
        glVertex2f(self.x + self.w + dx, self.y + 1 - self.h * (self.idx * self.spacing + 1) - dy)
        glTexCoord2f(0, 1 - (self.idx + 1) / 16.0)
        glVertex2f(self.x - dx, self.y + 1 - self.h * (self.idx * self.spacing + 1) - dy)
        glEnd()

class OptionsItem(Item):
    def __init__(self, x, y, w, h, idx, active):
        Item.__init__(self, x, y, w, h, idx)
        self.active = active
        if idx < 2:
            self.available = effects.sound_available
        else:
            self.available = True

    def redraw(self):
        dx, dy = self.selected and (0.2, 0.04) or (0, 0)
        # Choose toggle to show
        if self.idx == 3:
            tog = 3
        else:
            if self.idx < 2 and not self.available:
                tog = 2
            else:
                tog = int(self.active)
        glBegin(GL_QUADS)
        if self.selected:
            glColor4d(0, 213, 255, 255)
        else:
            glColor4f(0, 0.54, 0.74, 1)
        # Text
        glTexCoord2f(0.3125, 1 - (self.idx * 32) / 512.0)
        glVertex2f(self.x + self.w/5.0, self.y + 1 - self.h * self.idx * self.spacing + dy)
        glTexCoord2f(0.5625, 1 - (self.idx * 32) / 512.0)
        glVertex2f(self.x + self.w + dx, self.y + 1 - self.h * self.idx * self.spacing + dy)
        glTexCoord2f(0.5625, 1 - (self.idx + 1) * 32/512.0)
        glVertex2f(self.x + self.w + dx, self.y + 1 - self.h * (self.idx * self.spacing + 1) - dy)
        glTexCoord2f(0.3125, 1 - (self.idx + 1) * 32/512.0)
        glVertex2f(self.x + self.w/5.0, self.y + 1 - self.h * (self.idx * self.spacing + 1) - dy)

        glColor4f(1, 1, 1, 1)
        # Checkbox
        glTexCoord2f(0.3125+tog/16.0, 1 - 4 / 16.0)
        glVertex2f(self.x - dy, self.y + 1 - self.h * self.idx * self.spacing + dy)
        glTexCoord2f(0.375 +tog/16.0, 1 - 4 / 16.0)
        glVertex2f(self.x + self.w/5.0, self.y + 1 - self.h * self.idx * self.spacing + dy)
        glTexCoord2f(0.375 +tog/16.0, 1 - 5 / 16.0)
        glVertex2f(self.x + self.w/5.0, self.y + 1 - self.h * (self.idx * self.spacing + 1) - dy)
        glTexCoord2f(0.3125+tog/16.0, 1 - 5 / 16.0)
        glVertex2f(self.x - dy, self.y + 1 - self.h * (self.idx * self.spacing + 1) - dy)
        glEnd()

class Menu(object):
    def redraw(self):
        glDisable(GL_DEPTH_TEST)
        glDisable(GL_LIGHTING)
        glColor4f(1, 1, 1, 0.5)
        glEnable(GL_BLEND)
        glEnable(GL_TEXTURE_2D)
        clock = pygame.time.Clock()

        # Blank screen
        glClear(GL_COLOR_BUFFER_BIT)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        # Starry background
        glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_COLOR)
        glColor4f(1, 1, 1, 1)
        glBindTexture (GL_TEXTURE_2D, self.tex['menubg'].texid)
        glBegin(GL_QUADS)
        glTexCoord2f(0, 0)
        glVertex2f(-1.2, 2.2)
        glTexCoord2f(0, 1)
        glVertex2f(1.2, 2.2)
        glTexCoord2f(1, 1)
        glVertex2f(1.2, -1)
        glTexCoord2f(1, 0)
        glVertex2f(-1.2, -1)
        glEnd()
        glBindTexture (GL_TEXTURE_2D, self.tex['fonts'].texid)
        for i in self.items:
            i.redraw()
        pygame.display.flip()

    def run(self, width, height):
        self.width = width
        self.height = height
        self.playMusic()
        while not self.done:
            self.redraw()
            pygame.mouse.set_visible(True)
            event = pygame.event.wait()
            self.eventHandler(event)

    def eventHandler(self, event):
        if event.type == QUIT:
            self.done = True
        elif event.type == KEYDOWN:
            self.keyhandler(event)
        elif event.type == MOUSEMOTION:
            idx = self.idxFromPointer()
            if idx >= 0 and idx != self.selected:
                self.items[self.selected].selected = False
                self.selected = idx
                self.items[self.selected].selected = True
                effects.puc()
        elif event.type == MOUSEBUTTONDOWN:
            if pygame.mouse.get_pressed() == (True, False, False):
                idx = self.idxFromPointer()
                if idx >= 0:
                    effects.poc()
                    self.choiceHandler()

    def keyhandler(self, event):
        if event.key == K_RETURN or event.key == K_SPACE:
            effects.poc()
            self.choiceHandler()
        elif event.key == K_UP:
            self.items[self.selected].selected = False
            self.selected = (self.selected - 1) % self.nitems
            self.items[self.selected].selected = True
            effects.puc()
        elif event.key == K_DOWN:
            self.items[self.selected].selected = False
            self.selected = (self.selected + 1) % self.nitems
            self.items[self.selected].selected = True
            effects.puc()
        elif event.key == K_ESCAPE:
            self.done = True



class MainMenu(Menu):
    def __init__(self, tex):
        self.done = False
        self.tex = tex
        self.nitems = 5
        self.items = [Item(0.1, -0.05, 0.7, 0.17, x) for x in range(self.nitems)]
        self.selected = 0
        self.items[self.selected].selected = True

    def choiceHandler(self):
        if self.selected == 0:
            l = LevelSelector(self.tex)
            l.run(self.width, self.height)
        elif self.selected == 1:
            opt = OptionsMenu(self.tex)
            opt.run(self.width, self.height)
        elif self.selected == 3:
            about.main(self.width, self.height, self.tex)
        elif self.selected == 4:
            self.done = True

    def idxFromPointer(self):
        pos = pygame.mouse.get_pos()
        if pos[0] > 354 and pos[0] < 579 and pos[1] > 17 and pos[1] < 227:
            res = (pos[1] - 17) / 42
        else:
            res = -1
        return res
    def playMusic(self):
        effects.playMusic('menu')


class OptionsMenu(Menu):
    def __init__(self, tex):
        self.done = False
        self.nitems = 4
        self.tex = tex
        self.opts = ['music', 'sound', 'fx', 'back']
        self.items = [OptionsItem(0.1, -0.05, 0.7, 0.17, x, effects.fx.get(self.opts[x], True)) for x in range(4)]
        self.selected = 0
        self.items[self.selected].selected = True

    def choiceHandler(self):
        if self.selected == 3:
            effects.saveConfig()
            self.done = True
        else:
            if self.selected == 0: # Stop music (ignored if music is disabled)
                effects.stopMusic()
            self.items[self.selected].active = not self.items[self.selected].active
            effects.fx[self.opts[self.selected]] = self.items[self.selected].active
            if self.selected == 0: # Start music (ignored if music is disabled)
                effects.playMusic('menu')

    def idxFromPointer(self):
        pos = pygame.mouse.get_pos()
        if pos[0] > 354 and pos[0] < 579 and pos[1] > 17 and pos[1] < 227:
            res = min(self.nitems - 1, max(0, (pos[1] - 17) / 42))
        else:
            res = -1
        return res

    def playMusic(self):
        pass

    def idxFromPointer(self):
        pos = pygame.mouse.get_pos()
        if pos[0] > 354 and pos[0] < 579 and pos[1] > 10 and pos[1] < 177:
            res = (pos[1] - 17) / 42
        else:
            res = -1
        return res

class LevelItem(object):
    def __init__(self, menu):
        self.menu = menu
    def redraw(self):
        x = (self.menu.choice % 2) * 0.5
        y = 1 - (self.menu.choice / 2) * 0.33
        glBindTexture(GL_TEXTURE_2D, self.menu.tex['thumbnails'].texid)
        glBegin(GL_QUADS)
        # Background
        glColor4f(1, 1, 1, 0.8)
        glTexCoord2f(0.5, 0)
        glVertex2f(-0.42, 0.25)
        glTexCoord2f(1, 0)
        glVertex2f(0.42, 0.25)
        glTexCoord2f(1, 0.3333)
        glVertex2f(0.42, 0.95)
        glTexCoord2f(0.5, 0.3333)
        glVertex2f(-0.42, 0.95)

        glColor4f(1, 1, 1, 1)
        glTexCoord2f(x, y-0.3333)
        glVertex2f(-0.31, 0.35)
        glTexCoord2f(x+0.5, y-0.3333)
        glVertex2f(0.31, 0.35)
        glTexCoord2f(x+0.5, y)
        glVertex2f(0.31, 0.85)
        glTexCoord2f(x, y)
        glVertex2f(-0.31, 0.85)
        glEnd()
    def contains(self, pos):
        return False

class LabelItem(object):
    def __init__(self, menu):
        self.menu = menu
    def redraw(self):
        y = 0.63 - self.menu.choice * 0.04842
        glBindTexture(GL_TEXTURE_2D, self.menu.tex['fonts'].texid)
        glBegin(GL_QUADS)
        glColor4f(0, 0.54, 0.74, 1)
        glTexCoord2f(0, y-0.05)
        glVertex2f(-0.5, 0.025)
        glTexCoord2f(0.556, y - 0.05)
        glVertex2f(0.5, 0.025)
        glTexCoord2f(0.556, y)
        glVertex2f(0.5, 0.2)
        glTexCoord2f(0, y)
        glVertex2f(-0.5, 0.2)
        glEnd()
    def contains(self, pos):
        return False

class SelectorItem(object):
    pos = [[-0.7, 0.205, 0.15, 0.175, 0, 0.6875, 0.0625, 0.0625],
           [0.55, 0.205, 0.15, 0.175, 0.0625, 0.6875, 0.0625, 0.0625],
           [-0.7, -0.2, 1.4, 0.175, 0, 0.38, 0.556, 0.05]]
    def __init__(self, idx, menu):
        self.idx = idx
        self.menu = menu
    def redraw(self):
        x, y, w, h, u, v, wu, hv = self.pos[self.idx]
        dx, dy = (self.menu.selected == self.idx) and (w*0.15, h*0.15) or (0, 0)
        glBindTexture(GL_TEXTURE_2D, self.menu.tex['fonts'].texid)
        glBegin(GL_QUADS)
        if self.menu.selected == self.idx:
            glColor4d(0, 213, 255, 255)
        else:
            glColor4f(0, 0.54, 0.74, 1)
        glTexCoord2f(u, v-hv)
        glVertex2f(x-dx, y-h-dy)
        glTexCoord2f(u+wu, v-hv)
        glVertex2f(x+w+dx, y-h-dy)
        glTexCoord2f(u+wu, v)
        glVertex2f(x+w+dx, y+dy)
        glTexCoord2f(u,v)
        glVertex2f(x-dx, y+dy)
        glEnd()
    def contains(self, mousePos):
        x1 = self.pos[self.idx][0]*320+320
        x2 = (self.pos[self.idx][0] + self.pos[self.idx][2]) * 320 + 320
        y1 = 480 - (self.pos[self.idx][1]*240+240)
        y2 = 480 - (self.pos[self.idx][1]-self.pos[self.idx][3])*240+240
        return x1 < mousePos[0] and mousePos[0] < x2 and y1 < mousePos[1] and mousePos[1] < y2

class LevelSelector(Menu):
    def __init__(self, tex):
        self.done = False
        self.tex = tex
        self.choice = 0
        self.items = [LevelItem(self), LabelItem(self)]
        self.items += [SelectorItem(i, self) for i in range(3)]
        self.selected = -1

    def playMusic(self):
        pass

    def keyhandler(self, event):
        if event.key == K_RETURN or event.key == K_SPACE:
            effects.poc()
            self.choiceHandler()
        elif event.key == K_LEFT:
            if self.choice > 0:
                self.choice -= 1
                effects.puc()
        elif event.key == K_RIGHT:
            if self.choice < 2:
                self.choice += 1
                effects.puc()
        elif event.key == K_ESCAPE:
            self.done = True
        elif event.key == K_F6:
            tunnel.main(self.width, self.height, -10, 0) # Test enemies
            self.done = True
        elif event.key == K_F7:
            tunnel.main(self.width, self.height, -11, 0) # Test level
            self.done = True

    def choiceHandler(self):
        if self.selected == 0:
            if self.choice > 0:
                self.choice -= 1
                effects.poc()
        elif self.selected == 1:
            if self.choice < 2:
                self.choice += 1
                effects.poc()
        else:
            win = True
            level = self.choice
            score = 0
            while win == True and level < 3:
                win, score = tunnel.main(self.width, self.height, level, score)
                level += 1
            self.done = True
            effects.playMusic('menu')

    def idxFromPointer(self):
        pos = pygame.mouse.get_pos()
        for i in self.items:
            if i.contains(pos):
                return i.idx
        return -1

    def eventHandler(self, event):
        if event.type == QUIT:
            self.done = True
        elif event.type == KEYDOWN:
            self.keyhandler(event)
        elif event.type == MOUSEMOTION:
            idx = self.idxFromPointer()
            if idx >= 0 and idx != self.selected:
                effects.puc()
            if idx == -1:
                idx = 2
            self.selected = idx

        elif event.type == MOUSEBUTTONDOWN:
            if pygame.mouse.get_pressed() == (True, False, False):
                idx = self.idxFromPointer()
                if idx >= 0:
                    effects.poc()
                    self.choiceHandler()

