import pygame
from pygame.locals import *

from OpenGL.GL import *
from OpenGL.GLU import *

from random import random, choice
from texloader import LoadGLTextures

fireColors = [(0.98046875, 0.99609375, 0.3125), (0.99609375, 0.6640625, 0.08203125), (0.99609375, 0.1953125, 0.15625), (0.99609375, 0.9765625, 0.46484375)]

class Spark(object):
    def __init__(self, colors, speed=0.01, life=1.0):
        self.x, self.y, self.z = (0, 0, 0)
        self.spdx, self.spdy, self.spdz = (random() * speed - speed/2, random() * speed -speed / 2, random() * speed - speed / 2)
        self.life = life
        self.fade = random() * 0.1 + 0.03
        self.colors = colors
        self.r, self.g, self.b = self.genColor()
    def genColor(self):
        return choice(self.colors)

class Cloud(object):
    def __init__(self, pos, nparts, rad, tex, colors=fireColors, speed=0.01, life=1.0):
        self.pos = pos.coords()
        self.particles = [Spark(colors, speed=speed, life=life) for i in range(nparts)]
        self.texture = tex
        self.rad = rad

    def draw(self, world):
        if len(self.particles) > 0:
            glDepthMask(GL_FALSE)
            glMatrixMode(GL_MODELVIEW)
            glPushMatrix()
            i = 0
            glDisable(GL_LIGHTING)
            glEnable (GL_BLEND)
            glBlendFunc (GL_SRC_COLOR, GL_ONE)
            #glEnable(GL_TEXTURE_2D)
            glBindTexture(GL_TEXTURE_2D, self.texture['gradient'].texid)
            glTranslatef(self.pos[0], self.pos[1], self.pos[2])
            s = world.hero.section
            m = list(s.right.coords()) + [0] + list(s.up.coords()) + [0] + list(s.forward.coords()) + [0, 0, 0, 0, 1]
            glMultMatrixf(m)
            while i < len(self.particles):
                p = self.particles[i]
                glColor4f(p.r, p.g, p.b, p.life)
                #glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION, (p.r, p.g, p.b, p.life))
                glBegin(GL_QUADS)
                glTexCoord2f(0, 0)
                glVertex3f(p.x - self.rad, p.y - self.rad, p.z)
                glTexCoord2f(0, 1)
                glVertex3f(p.x - self.rad, p.y + self.rad, p.z)
                glTexCoord2f(1, 1)
                glVertex3f(p.x + self.rad, p.y + self.rad, p.z)
                glTexCoord2f(1, 0)
                glVertex3f(p.x + self.rad, p.y - self.rad, p.z)
                glEnd()
                p.life -= p.fade
                if p.life < 0:
                    del self.particles[i]
                else:
                    p.x += p.spdx
                    p.y += p.spdy
                    p.z += p.spdz
                    i = i + 1
            glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION, (0, 0, 0, 1))
            glEnable(GL_LIGHTING)
            glPopMatrix()
            glDepthMask(GL_TRUE)
            return len(self.particles) == 0

class ParticleEngine(object):
    def __init__(self):
        self.texture = None
        self.clouds = []
    def pump (self, world):
        i = 0
        while i < len(self.clouds):
            if self.clouds[i].draw(world):
                # draw() returns True when the cloud has vanished
                del self.clouds[i]
            else:
                i = i + 1
    def addCloud(self, pos, n, rad, **kw):
        if self.texture is None:
            self.texture = LoadGLTextures({'gradient': "gradient.bmp"})
        self.clouds.append(Cloud(pos, n, rad, self.texture, **kw))

