from OpenGL.GL import *
from OpenGL.GLU import *

from texloader import GameTexture
from vector import Vector
from math import pi, sin, cos

class Section(object):
    defaultRadius = 1
    sides = 8
    def __init__ (self, pos, prev, id=None):
        self.pos = pos
        self.prev = prev
        self.next = None
        self.rad = self.defaultRadius
        self.color = (1,1,1)
        self.texture = GameTexture('invalid', 0)
        if prev:
            prev.next = self
        if id is not None:
            self.id = id
        elif prev:
            self.id = prev.id + 1
        else:
            self.id = 0

    def setPrev(self, newprev):
        self.prev = newprev
        if newprev:
            newprev.next = self
            s = self
            while s != s.next:
                s.id = s.prev.id + 1

    def contains (self, point):
        normforward = self.right.cross (self.up)
        delta = point - self.pos
        return normforward.dot(delta) >= 0

    def length(self):
        return (self.next.pos - self.pos).norm()

    def updateCoords (self):
        f = Vector(0, 0, 0)
        if self.prev:
            f += (self.pos - self.prev.pos).normalize()
        if self.next:
            f += (self.next.pos - self.pos).normalize()
        self.forward = f.normalize()
        if not self.prev:
            self.up = Vector (0, 1, 0)
        else:
            proj = self.forward * self.prev.up.dot(self.forward)
            self.up = (self.prev.up - proj).normalize()
        self.right = self.up.cross (self.forward)
        if self.next:
            self.path_vector = self.next.pos -self.pos
        else:
            self.path_vector = self.prev.path_vector
        self.path_vector = self.path_vector.normalize()
        self.apotema = self.rad * cos(pi / self.sides)

    def calculatePoints (self):
        self.points = []
        self.normals = []
        for i in range(self.sides):
            self.points.append (self.pos + self.right * self.rad * cos (2*pi*i / self.sides) +
                                           self.up * self.rad * sin (2*pi*i / self.sides))
            self.normals.append (- self.right * cos (2*pi*i / self.sides) - self.up * sin (2*pi*i / self.sides))

    def draw (self):
        texpoints = [(0, 0), (0, 10), (5, 0), (5, 10)]
        idx = 0
        cant = 0.0
        n = len(self.points)

        blend = self.texture.texid <> self.next.texture.texid
        # TODO: Enable blending
        if blend:
            glEnable(GL_COLOR_MATERIAL)
            glEnable(GL_BLEND)
            glBlendFunc(GL_SRC_ALPHA, GL_ZERO)
            # Wireframe view for debugging
            # glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
        
        glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, (1, 1, 1, 1))
        glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, (0, 0, 0, 1))
        glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION, (0, 0, 0, 1))
        
        glBindTexture(GL_TEXTURE_2D, self.texture.texid)
        glBegin(GL_QUAD_STRIP)
        for a, an, b, bn in zip(self.points, self.normals, self.next.points, self.next.normals):
            glColor4fv(self.color+(1,))
            #glTexCoord2f(texpoints[idx][0], texpoints[idx][1])
            glTexCoord2f(cant/n, 0)
            glNormal3fv (an.coords())
            glVertex3fv(a.coords())
            idx += 1
            glColor4fv(self.next.color+(0,))
            #glTexCoord2f(texpoints[idx][0], texpoints[idx][1])
            glTexCoord2f(cant/n, 1)
            glNormal3fv (bn.coords())
            glVertex3fv(b.coords())
            idx += 1
            idx %= len(texpoints)
            cant += 1
        glColor4fv(self.color+(1,))
        #glTexCoord2f(texpoints[idx][0], texpoints[idx][1])
        glTexCoord2f(1, 0)
        glNormal3fv (self.normals[0].coords())
        glVertex3fv(self.points[0].coords())
        idx += 1
        glColor4fv(self.next.color+(0,))
        #glTexCoord2f(texpoints[idx][0], texpoints[idx][1])
        glTexCoord2f(1, 1)
        glNormal3fv (self.next.normals[0].coords())
        glVertex3fv(self.next.points[0].coords())
        glEnd()

        # Second pass for texture-blending
        if blend:
            glDepthFunc(GL_LEQUAL)
            glBlendFunc(GL_SRC_ALPHA, GL_ONE)
            idx = 0
            cant = 0.0
            glBindTexture(GL_TEXTURE_2D, self.next.texture.texid)
            glBegin(GL_QUAD_STRIP)
            for a, an, b, bn in zip(self.points, self.normals, self.next.points, self.next.normals):
                glColor4fv(self.color+(0,))
                glTexCoord2f(cant/n, 0)
                #glTexCoord2f(texpoints[idx][0], texpoints[idx][1])
                glNormal3fv (an.coords())
                glVertex3fv(a.coords())
                idx += 1
                glColor4fv(self.next.color+(1,))
                glTexCoord2f(cant/n, 1)
                #glTexCoord2f(texpoints[idx][0], texpoints[idx][1])
                glNormal3fv (bn.coords())
                glVertex3fv(b.coords())
                idx += 1
                idx %= len(texpoints)
                cant += 1
            glColor4fv(self.color+(0,))
            #glTexCoord2f(texpoints[idx][0], texpoints[idx][1])
            glTexCoord2f(1, 0)
            glNormal3fv (self.normals[0].coords())
            glVertex3fv(self.points[0].coords())
            idx += 1
            glColor4fv(self.next.color+(1,))
            #glTexCoord2f(texpoints[idx][0], texpoints[idx][1])
            glTexCoord2f(1, 1)
            glNormal3fv (self.next.normals[0].coords())
            glVertex3fv(self.next.points[0].coords())
            glEnd()
            glDisable(GL_BLEND)
            glDisable(GL_COLOR_MATERIAL)
        
# Uncomment this to show the center of each section in black
        #if self.next:
        #    glEnable (GL_LINE_SMOOTH)
        #    glLineWidth (2)
        #    glBegin (GL_LINES)
        #    glVertex3fv(self.pos.coords())
        #    glVertex3fv(self.next.pos.coords())
        #    glEnd()
        #glDisable (GL_LINE_SMOOTH)

class FirstSection(object):
    pass

class LastSection(object):
    def __init__ (self, prev):
        self.prev = prev
        self.next = self
        if prev:
            prev.next = self
        self.pos = prev.pos
        self.sides = prev.sides
        self.rad = prev.rad
        self.color = prev.color
        self.id = prev.id + 1
        self.texture = GameTexture('invalid', 0)

    def length(self):
        return 1

    def updateCoords(self):
        self.right = self.prev.right
        self.up = self.prev.up
        self.forward = self.prev.forward
        self.path_vector = self.prev.prev.path_vector
        self.apotema = self.prev.apotema

    def calculatePoints (self):
        self.points = []
        self.normals = []
        for i in range(self.sides):
            self.points.append (self.pos + self.right * self.rad * cos (2*pi*i / self.sides) +
                                           self.up * self.rad * sin (2*pi*i / self.sides))
            self.normals.append (- self.right * cos (2*pi*i / self.sides) - self.up * sin (2*pi*i / self.sides))
    def contains (self, point):
        return self.prev.contains (point)

    def draw (self):
        pass
