import effects

class Mover(object):
    """Base class for ships, shots, and other stuff that moves around"""
    def wouldCollideWithWalls (self, newpos, radius = None):
        """ Returns True if newpos is outside the tunnel """
        if not radius: radius = self.radius
        sec = self.section
        if sec.path_vector.norm() == 0:
            return False
        d = sec.path_vector.cross(sec.pos - newpos).norm() / sec.path_vector.norm()
        return d > self.section.apotema - self.radius

    def adjust (self):
        """update current section when we got too far. This breaks a lot if
        you get too far away from the tunnel; i.e., farther than the
        intersection of the section planes for the endpoints of the
        current section. It might break also if you move too far away forward
        or backwards."""
        while self.section and not self.section.contains(self.pos):
            self.section = self.section.prev
        while self.section and self.section.next.contains(self.pos) and self.section.next is not self.section:
            self.section = self.section.next
            self.dist = 0

class Ship(Mover):
    defaultRadius = 0.14
    collisionDamage = 0.0
    def __init__ (self, section):
        """New ship, moving along section. initial position is in the center
        of the section beginning"""
        self.section = section
        self.pos = section.pos
        self.radius = self.defaultRadius
        self.dist = 0

    def forward (self, distance):
        """Move along current section by given distance (can be negative to move
        backards)"""
        self.pos += self.section.path_vector * distance
        self.dist += distance / self.section.length()
        self.adjust()

    def takeDamage (self, damage):
        pass

    def collidedWall (self):
        pass

    def strafe (self, vx, vy):
        """Move perpendicular to the section, following the given 2D vector.
        (0,1) means "up" (each section has its "up" direction)"""
        sec = self.section
        newpos = self.pos + sec.right * vx + sec.up * vy
        if self.wouldCollideWithWalls (newpos):
            self.collidedWall()
        else:
            self.pos = newpos
        self.adjust()

