from OpenGL.GL import *
from OpenGL.GLU import *

from ship import Mover
import effects

class Shot(Mover):
    length = 0.1
    speed = 0.09
    power = 0.6 # Tka into account that these are usually shot in pairs
    def __init__ (self, pos, direction, section, friendly=False):
        self.pos = pos
        self.direction = direction.normalize()
        self.section = section
        self.radius = 0
        self.dead = False
        if friendly:
            self.check_hits = self.check_hits_enemies
        else:
            self.check_hits = self.check_hits_hero

    def move (self):
        newpos = self.pos + (self.direction * self.speed)
        if self.wouldCollideWithWalls (newpos):
            self.dead = True
            effects.laserBounce(self.pos)
        else:
            self.pos = newpos
        self.adjust()

    def customfx(self):
        """special fx for enemy hit"""
        effects.enemyDamageSparks (self.pos)

    def check_hits_enemies(self, world):
        for e in world.enemies:
            d = (self.pos - e.pos).norm()
            if d < e.radius:
                e.hit (self.power)
                self.customfx()
                self.dead = True

    def check_hits_hero(self, world):
        e = world.hero
        d = (self.pos - e.pos).norm()
        if d < e.radius:
            e.takeDamage(0.3)
            self.dead = True

    def draw (self):
        glEnable (GL_LINE_SMOOTH)
        glEnable(GL_BLEND)
        glBlendFunc (GL_SRC_COLOR, GL_ONE)
        glBindTexture (GL_TEXTURE_2D, 0)
        glLineWidth(4)
        glBegin(GL_LINES)
        glColor3fv((1, 0, 0))
        glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT, (0, 0, 0, 0))
        glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, (0, 0, 0, 0))
        glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION, (1, 0, 0, 0))
        glVertex3fv(self.pos.coords())
        glVertex3fv((self.pos + self.direction * self.length).coords())
        glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT, (0.2, 0.2, 0.2, 1))
        glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, (0.8, 0.8, 0.8, 0))
        glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION, (0, 0, 0, 1))
        glEnd()
        glDisable (GL_LINE_SMOOTH)
        glDisable(GL_BLEND)

    def adjust(self):
        Mover.adjust(self)
        if self.section is None: self.dead = True
        

class PlasmaBlast(Shot):
    size = 0.04
    speed = 0.035
    power = 5

    def __init__ (self, pos, direction, section, world, texture, friendly=False):
        Shot.__init__ (self, pos, direction, section, friendly)
        self.rotation = 0
        self.texture = texture
        self.world = world
        if friendly:
            self.speed *= 4

    def move (self):
        Shot.move(self)
        self.rotation += 10

    def customfx(self):
        """special fx for enemy hit"""
        effects.plasmaExplosion (self.pos)

    def draw (self):
        glEnable(GL_BLEND)
        glBlendFunc (GL_ONE, GL_ONE_MINUS_SRC_COLOR)
        glBindTexture (GL_TEXTURE_2D, self.texture.texid)
        glDepthMask(GL_FALSE)
        glPushMatrix()
        h = self.world.hero
        glTranslatef(self.pos.x, self.pos.y, self.pos.z)
        m = list(h.section.right.coords()) + [0] + list(h.section.up.coords()) + [0] + list(h.section.path_vector.coords()) + [0, 0, 0, 0, 1]
        glMultMatrixf(m)
        glRotatef (self.rotation, 0,0,1) 

        glBegin(GL_QUADS)
        glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION, (1, 1, 1, 0))
        glTexCoord2f(0, 0)
        glVertex3fv((-self.size,-self.size,0)) 
        glTexCoord2f(1, 0)
        glVertex3fv((self.size,-self.size,0)) 
        glTexCoord2f(1, 1)
        glVertex3fv((self.size,self.size,0)) 
        glTexCoord2f(0, 1)
        glVertex3fv((-self.size,self.size,0))        
        glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION, (0, 0, 0, 1))
        glEnd()
    
        glPopMatrix()
    
        glDisable(GL_BLEND)
        glDepthMask(GL_TRUE)

