from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
from math import pi, sin, cos, sqrt
from data import filepath
from vector import Vector
from section import Section
from world import World
from texloader import LoadGLTextures
import effects

def main(width, height, level, startingScore):
    """A certain level"""
    textures = LoadGLTextures ({'win': "win.bmp",
                                'lose': "lose.bmp",
                                'level0': "level0.bmp",
                                'level1': "level1.bmp",
                                'level2': "level2.bmp",
                                'level3': "finale.bmp"}) #The final hack

    glEnable(GL_TEXTURE_2D)                    # Enable texture mapping.
    showIntro(level, textures);
    
    glEnable(GL_DEPTH_TEST)
    glMatrixMode(GL_PROJECTION)
    fovy = 90.0  # Field-of-view angle
    glLoadIdentity()
    gluPerspective(fovy, width/height, 0.1, 40.0)    #setup lens
    glMatrixMode(GL_MODELVIEW)
    glEnable(GL_TEXTURE_2D)                    # Enable texture mapping.

    lightAmbient = [0.3, 0.3, 0.3, 1]
    lightDiffuse = [1, 1, 1, 1]
    lightSpecular = [1, 1, 1, 1]
    lightPosition = [0, 0, 0, 1]

    glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient)
    glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse)
    glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular)
    glLightfv(GL_LIGHT0, GL_POSITION, lightPosition)

    glEnable(GL_LIGHT0)
    glDepthFunc(GL_LESS)

    glEnable(GL_LIGHTING)
    pygame.mouse.set_visible (False)
    levelMusic = 'level%d' % level
    levelFile = 'levels/level%d.txt' % level
    if level == -10:
        levelFile = 'levels/testlevel.txt'
        levelMusic = 'level0'
    elif level == -11:
        levelFile = 'levels/enemy_test.txt'
        levelMusic = 'level0'

    effects.playMusic(levelMusic)
    w = World(fovy, width, height, levelFile)
    w.score = startingScore
    if level==1:
        w.MAX_SECTIONS = 10
    if level==2:
        w.hero.health *= 1.1 # Balancing magic :)
        w.hero.fullHealth *= 1.1 # Balancing magic :)
    w.run()
    if w.won or w.lost:
        showResults(w, textures)
    if level == 2 and w.won:
        showIntro(3, textures)
    return (w.won, w.score)

def showIntro(level, textures):
    if level < 0 or level > 3: return
    glDisable(GL_DEPTH_TEST)
    glDisable(GL_LIGHTING)
    glDisable(GL_BLEND)
    glEnable(GL_TEXTURE_2D)
    glColor4f(1, 1, 1, 1)

    # Blank screen
    glClear(GL_COLOR_BUFFER_BIT)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    
    # Draw summary screen
    glColor4f(1, 1, 1, 1)
    glBindTexture (GL_TEXTURE_2D, textures['level'+str(level)].texid)
    glBegin(GL_QUADS)
    glTexCoord2f(0, 0)
    glVertex2f(-1, -1)
    glTexCoord2f(0, 1)
    glVertex2f(-1, 1)
    glTexCoord2f(1, 1)
    glVertex2f(1, 1)
    glTexCoord2f(1, 0)
    glVertex2f(1, -1)
    glEnd()
    pygame.display.flip()
    
    done = False
    while not done:
        event = pygame.event.wait()
        if event.type == QUIT or (event.type == KEYDOWN and event.key == K_ESCAPE) or (event.type == KEYDOWN and event.key == K_RETURN):
            done = True
    

def showResults(w, textures):
    
    chars = {'0': [0, 0.085],
            '1': [0.09, 0.06],
            '2': [0.15, 0.09],
            '3': [0.24, 0.085],
            '4': [0.33, 0.09],
            '5': [0.42, 0.09],
            '6': [0.51, 0.085],
            '7': [0.59, 0.08],
            '8': [0.675, 0.085],
            '9': [0.77, 0.09],
            '-': [0.94, 0.05]}
    
    if w.won:
        print "Win :)"
        background = textures['win']
    elif w.lost:
        background = textures['lose']
        print "Lost :("
    #else:
        #print "You quit..."
    
    # We're no longer in 3D mode
    glDisable(GL_DEPTH_TEST)
    glDisable(GL_LIGHTING)
    glDisable(GL_BLEND)
    glEnable(GL_TEXTURE_2D)
    glColor4f(1, 1, 1, 1)

    # Blank screen
    glClear(GL_COLOR_BUFFER_BIT)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    glScalef(1, 1, 1)
    
    # Draw summary screen
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_COLOR)
    glColor4f(1, 1, 1, 1)
    glBindTexture (GL_TEXTURE_2D, background.texid)
    glBegin(GL_QUADS)
    glTexCoord2f(0, 0)
    glVertex2f(-1, -1)
    glTexCoord2f(0, 1)
    glVertex2f(-1, 1)
    glTexCoord2f(1, 1)
    glVertex2f(1, 1)
    glTexCoord2f(1, 0)
    glVertex2f(1, -1)
    glEnd()
    
    #Draw score
    glBindTexture (GL_TEXTURE_2D, w.textures['gauge'].texid)
    glEnable(GL_BLEND)
    glScale(2, -2, 1)
    if w.won:
        y = 0.33
    else:
        y = 0.295
    w.drawScore(0.08, y, 0, 0.04, 0.0, w.score)
    pygame.display.flip()
    
    done = False
    while not done:
        event = pygame.event.wait()
        if event.type == QUIT or (event.type == KEYDOWN and event.key == K_ESCAPE) or (event.type == KEYDOWN and event.key == K_RETURN):
            done = True

if __name__ == '__main__': main()
