from math import sqrt, cos, sin
class Vector (object):
    def __init__ (self, x, y, z):
        self.x = x
        self.y = y
        self.z = z
    def coords(self):
        return (self.x, self.y, self.z)
    def __add__(self, other):
        return Vector(self.x + other.x, self.y + other.y, self.z + other.z)
    def __sub__(self, other):
        return Vector(self.x - other.x, self.y - other.y, self.z - other.z)
    def dot(self, other):
        return sum ([x * y for x, y in zip(self.coords(), other.coords())])
    def __mul__ (self, k):
        return Vector (self.x*k, self.y*k, self.z*k)
    def __div__ (self, k):
        return Vector (self.x/k, self.y/k, self.z/k)
    def __neg__(self):
        return Vector(-self.x, -self.y, -self.z)
    def norm(self):
        return sqrt(self.dot(self))
    def cross (self, other):
        return Vector (self.y * other.z - self.z * other.y,
                       self.z * other.x - self.x * other.z,
                       self.x * other.y - self.y * other.x)
    def normalize (self):
        norm = self.norm()
        if norm == 0: norm = 1
        return Vector(self.x/norm, self.y/norm, self.z/norm)
    def rotate (self, angle, axis):
        c = cos(angle); s = sin(angle); C = 1-c
        xs = axis.x*s;   ys = axis.y*s;   zs = axis.z*s
        xC = axis.x*C;   yC = axis.y*C;   zC = axis.z*C
        xyC = axis.x*yC; yzC = axis.y*zC; zxC = axis.z*xC
        M1 = Vector(axis.x*xC+c,   xyC-zs,   zxC+ys)
        M2 = Vector(xyC+zs,   axis.y*yC+c,   yzC-xs)
        M3 = Vector(zxC-ys,   yzC+xs,   axis.z*zC+c)
        return Vector (self.dot(M1),self.dot(M2),self.dot(M3))
    def __str__ (self):
        return "Vector(%f,%f,%f)" % (self.x, self.y, self.z)
    def __repr__ (self):
        return "Vector(%s,%s,%s)" % (repr(self.x), repr(self.y), repr(self.z))
