#!BPY

"""
Name: 'Raw Triangle And Texture (.rat)...'
Blender: 232
Group: 'Export'
Tooltip: 'Export selected mesh to Raw Triangle And Texture Format (.rat)'
"""

__author__ = "Anthony Lenton"
__bpydoc__ = """\
This script exports meshes to Raw Triangle And Texture file format.

Each triangle is printed on a set of four lines, the first representing
the triangle's normal, and one more for each vertex.  If the vertex is
UV-Mapped then it also includes that information.

This format doesn't exist, it was concocted up for our pyweek5's entry.

Usage:<br>
    Select meshes to be exported and run this script from "File->Export" menu.
"""

#
# Based on Anthony D'Agostino's Raw Triangle Exporter
#
# To use this script, place it in /usr/lib/blender/scripts/ or in whichever
# directory your blender scripts go in!

# ***** BEGIN GPL LICENSE BLOCK *****
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import Blender, meshtools
import sys

# =============================================
# === Write Raw Triangle and Texture Format ===
# =============================================

def write(filename):
    file = open(filename, "w")

    objects = Blender.Object.GetSelected()
    objname = objects[0].name
    meshname = objects[0].data.name
    mesh = Blender.NMesh.GetRaw(meshname)
    obj = Blender.Object.Get(objname)
    data = obj.getData()

    std=sys.stdout
    sys.stdout=file
    for face in mesh.faces:
        triangs = [range(3)]
        if len(face.v) == 4:
            triangs.append([2, 3, 0])
        for t in triangs:
            print "%f %f %f" % tuple(face.normal)
            for i in t:
                uvs = ""
                if data.hasFaceUV():
                    uvs = " %f %f" % tuple(face.uv[i])
                print "%f %f %f%%s" % tuple(face.v[i]) % uvs
    sys.stdout=std


    Blender.Window.DrawProgressBar(1.0, '')  # clear progressbar
    file.close()
    message = "Successfully exported " + Blender.sys.basename(filename)
    meshtools.print_boxed(message)

def fs_callback(filename):
    if filename.find('.rat', -4) <= 0: filename += '.rat'
    write(filename)

Blender.Window.FileSelector(fs_callback, "Export Rat")
